###################################################
# This script takes user through each sound file in a directory
# and allows for adjustment of formant and/or pitch settings.
# Final choices are recorded in output. 
#
# This output can then be used as the basis for future 
# formant/pitch measurement scripts to minimize errors. 
#
# User specifies default/baseline parameters.
# If different baselines are desired for different participants,
# the script can be run separately for each subset with different
# defaults. 
# 
# Ideally, settings should be chosen that have no errors
# throughout any region that will be measured in the future.
# However, if this is impossible, a "notes" field allows
# user to note this or any other comments. 
#
# This assumes a corresponding TG file for each wav file.
# if tgSave==T, it will save the TGs as it goes through,
# so annotations also can be corrected if necessary.
#
# sample_OK_measures.txt provides parameters that I thought were OK 
# for these data (considering only the annotated vowels for formants). 
#
# jessamyn.schertz@utoronto.ca
# November 2022
###################################################

## USER INPUT #####################################

beginPause: "User input"
	word: "datadir", "data"
	word: "output", "parameters_formant_f0.txt"
	comment: "Check f0? If so, set default parameters"
	boolean: "check_f0", 1
	positive: "f0_floor_default", 100
	positive: "f0_ceiling_default", 600
	comment: "Check formants? If so, set default parameters"
	boolean: "check_formants", 1
	positive: "max_num_formants_default", 4.5
	positive: "formant_ceiling_default", 5000
	comment: "Save TextGrids (if modifications have been made)?"
	boolean: "tgSave", 1
	comment: "Write parameters to output file? (if no, just allows for checking)"
	boolean: "writeFile", 1
endPause: "Continue", 1

if right$(datadir$, 1) <> "/" and datadir$ <> ""
	datadir$ = datadir$ + "/"
endif

if writeFile == 1
	@make.header
endif

## MAIN LOOP #####################################

files = Create Strings as file list: "fileList", datadir$+"*.wav"
num.files = Get number of strings
for file from 1 to num.files
	select files
	filename$ = Get string: file
	filename$ = filename$ - ".wav"

	sound = Read from file: datadir$+filename$+".wav"
	tg = Read from file: datadir$+filename$+".TextGrid"
	
	@check
	
	select tg
	if tgSave == 1
		Save as text file: datadir$+filename$+".TextGrid"
	endif
	plus sound
	Remove
endfor


## PROCEDURES #####################################

procedure check

	f0.floor$ = "NA"
	f0.ceiling$ = "NA"
	f0.notes$ = "NA"
	max.num.formants$ = "NA"
	formant.ceiling$ = "NA"
	formant.notes$ = "NA"

	select tg
	plus sound
	View & Edit
	editor: tg

		f0.floor.old = f0_floor_default
		f0.ceiling.old = f0_ceiling_default
		Pitch settings: f0.floor.old, f0.ceiling.old, "Hertz", "autocorrelation", "automatic"

		formant.ceiling.old = formant_ceiling_default
		max.num.formants.old = max_num_formants_default
		Formant settings: formant.ceiling.old, max.num.formants.old, 0.025, 30, 1


		if check_f0 == 1
			@check.f0
		endif
		if check_formants == 1
			@check.form
		endif
			
		Close
	endeditor

	if writeFile == 1
		@write.to.output
	endif

endproc

procedure check.f0
	f0.floor.old = f0_floor_default
	f0.ceiling.old = f0_ceiling_default
	Pitch settings: f0.floor.old, f0.ceiling.old, "Hertz", "autocorrelation", "automatic"

	beginPause: "Check f0"
		comment: "Turn on pitch tracking if necessary."
		comment: "Change parameter settings if needed."
		comment: "When you are happy with the settings, press Continue."
		comment: "If you can't find a good solution, make a note." 
		positive: "f0.floor", f0.floor.old
		positive: "f0.ceiling", f0.ceiling.old
		word: "f0.notes", "NA"
	clicked = endPause: "Continue", 1
	while f0.floor <> f0.floor.old or f0.ceiling <> f0.ceiling.old
		f0.floor.old = f0.floor
		f0.ceiling.old = f0.ceiling
		Pitch settings: f0.floor.old, f0.ceiling.old, "Hertz", "autocorrelation", "automatic"

		beginPause: "Check f0"
			comment: "Turn on pitch tracking if necessary."
			comment: "Change parameter settings if needed."
			comment: "When you are happy with the settings, press Continue."
			comment: "If you can't find a good solution, make a note." 
			positive: "f0.floor", f0.floor.old
			positive: "f0.ceiling", f0.ceiling.old
			word: "f0.notes", f0.notes$
		clicked = endPause: "Continue", 1
	endwhile

	f0.floor$ = "'f0.floor'"
	f0.ceiling$ = "'f0.ceiling'"
endproc

procedure check.form
	formant.ceiling.old = formant_ceiling_default
	max.num.formants.old = max_num_formants_default
	Formant settings: formant.ceiling.old, max.num.formants.old, 0.025, 30, 1

	beginPause: "Check Formants"
		comment: "Turn on formant tracking if necessary."
		comment: "Change parameter settings if needed."
		comment: "When you are happy with the settings, press Continue."
		comment: "If you can't find a good solution, make a note." 
		positive: "formant.ceiling", formant.ceiling.old
		positive: "max.num.formants", max.num.formants.old
		word: "formant.notes", "NA"
	clicked = endPause: "Continue", 1
	while formant.ceiling <> formant.ceiling.old or max.num.formants <> max.num.formants.old
		formant.ceiling.old = formant.ceiling
		max.num.formants.old = max.num.formants
		Formant settings: formant.ceiling.old, max.num.formants.old, 0.025, 30, 1

		beginPause: "Check Formants"
			comment: "Turn on formant tracking if necessary."
			comment: "Change parameter settings if needed."
			comment: "When you are happy with the settings, press Continue."
			comment: "If you can't find a good solution, make a note." 
			positive: "formant.ceiling", formant.ceiling.old
			positive: "max.num.formants", max.num.formants.old
			word: "formant.notes", formant.notes$
		clicked = endPause: "Continue", 1
	endwhile

	formant.ceiling$ = "'formant.ceiling'"
	max.num.formants$ = "'max.num.formants'"
endproc


procedure make.header
	header$ = "filename	f0.floor	f0.ceiling	f0.notes	"
	header$ = header$ + "num.form	max.form	form.notes"	
	writeFileLine: output$, header$
endproc

procedure write.to.output
	new$ = filename$+tab$+f0.floor$+tab$+f0.ceiling$+tab$+f0.notes$+tab$
	new$ = new$+max.num.formants$+tab$+formant.ceiling$+tab$+formant.notes$
	appendFileLine: output$, new$
endproc


